#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#define F_CPU 16000000UL

#define connected 1
#define unconnected 0

// tipurile de cabluri
int straight[8] = { 1, 2, 3, 4, 5, 6, 7, 8 };
int crossover_T568A[8] = { 3, 6, 1, 7, 8, 2, 4, 5 };
int crossover_T568B[8] = { 3, 6, 1, 4, 5, 2, 7, 8 };
int rollover[8] = { 8, 7, 6, 5, 4, 3, 2, 1 };
int tested_cable[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };

int status = unconnected;

// functie care compara 2 vectori
int compare_function (int *x, int *y) {

	int i;
	
	for (i = 0; i < 8; i++) {
	
		if (x[i] != y[i]) {
		
			return 0;
		}
	}
	
	return 1;
}

// returneaza bitul setat din registrul primit ca parametru
int get_set_bit (int reg)
{
	
	int i;
	
	for (i = 0; i < 8; i++)
	
		if (bit_is_set (reg, i))
		
			return i + 1;
			
	return 0;

}

// verifica daca este inserat cablul
int cable_in () 
{

	int i, reg;
	
	for (i = 0; i < 8; i++) {
	
		PORTD = 1 << i;
		
		reg = PINC;
		
		if (get_set_bit (reg) != 0) {
		
			PORTD = 0x00;
			
			return 1;
		}
		
	}
	
	//PORTD = 0x00;
	
	return 0;
	
}

// functie care determina ce fel de cablu este si va aprinde
// LED-ul corespunzator
void get_cable_type ()
{

	if (compare_function (tested_cable, straight) == 1)
		PORTD = 1 << 0;
	else 
		if (compare_function (tested_cable, crossover_T568A) == 1)
			PORTD = 1 << 1;
	else 
		if (compare_function (tested_cable, crossover_T568B) == 1)
			PORTD = 1 << 2;
			
	else
		if (compare_function (tested_cable, rollover) == 1)
			PORTD = 1 << 3;
	else
		PORTD = 1 << 4;

}

int main()
{		

	int i, k;
	int reg;
	
	// setez directia pinilor porturilor (input sau output)
	DDRA = 0xFF; // output => sender
	DDRC = 0x00; // input => receiver
	DDRD = 0xFF; // output => LED'S
	
	
	
	while (1) {
	
		status = unconnected;  // cablul nu e conectat initial
		
		if (cable_in()) {	  // verific daca e inserat cablul
		
			PORTD = 0x00;	// LED-urile nu arata nimic momentan
		
			for (i = 0; i < 8; i++) {  
			
				// se trimite semnalul pe fiecare pin
				PORTA = 1 << i;
				
				// wait for it to arrive
				_delay_ms(500);
					
				// ce a primit receiver-ul
				reg = PINC;
				
				// afla bitul setat/activ
				tested_cable[i] = get_set_bit (reg);
				
				if (tested_cable[i] != 0) { // verifica daca cel putin un fir e conectat			
					
					// daca nu era conectat cablul, setam statusul ca fiind conectat
					if (status == unconnected)
						status = connected;
						
			
				}
				
				else {
				
					// cablul nu e conectat
					status = unconnected;
					
					PORTD = 1 << 4;
				
					break;
					
				}
			
			}
			
			
			PORTA = 0x00;	// sender-ul nu mai trimite nimic
			
			if (status == connected) {  // cablul este conectat
			
				get_cable_type();  // afla tipul cablului
				
				_delay_ms(2500);
				
				status = unconnected;  // reseteaza statusul
			
			}
			
			
		}
		
		else {
			
			// eroare, cablul nu e inserat
			
			//PORTD = 1 << 4;
			
			// verificare ca se intampla ceva; 
			
			for (k = 0; k < 11; k++) {
			
				PORTD = PORTD ^ 0xFF;
			
				_delay_ms(1000);
				
			}
			
				
		}
			
	
	}

	return 0;
}